@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 Igalia S.L.
+ * Copyright (C) 2010, 2011 Igalia S.L.
  *
  * Contact: Iago Toral Quiroga <itoral@igalia.com>
  *
@@ -281,7 +281,7 @@ grl_media_serialize_extended (GrlMedia *media,
   GRegex *type_regex;
   GString *serial;
   GrlKeyID grlkey;
-  GrlPluginRegistry *registry;
+  GrlRegistry *registry;
   const GValue *value;
   const gchar *id;
   const gchar *source;
@@ -296,8 +296,8 @@ grl_media_serialize_extended (GrlMedia *media,
   /* Check serialization type */
   switch (serial_type) {
   case GRL_MEDIA_SERIALIZE_FULL:
-    registry = grl_plugin_registry_get_default ();
-    keylist = grl_plugin_registry_get_metadata_keys (registry);
+    registry = grl_registry_get_default ();
+    keylist = grl_registry_get_metadata_keys (registry);
     serial_media = grl_media_serialize_extended (media,
                                                  GRL_MEDIA_SERIALIZE_PARTIAL,
                                                  keylist);
@@ -334,12 +334,12 @@ grl_media_serialize_extended (GrlMedia *media,
     /* Include all properties */
     if (serial_type == GRL_MEDIA_SERIALIZE_PARTIAL) {
       g_string_append_c (serial, '?');
-      registry = grl_plugin_registry_get_default ();
+      registry = grl_registry_get_default ();
 
       va_start (va_serial, serial_type);
       keylist = va_arg (va_serial, GList *);
       for (key = keylist; key; key = g_list_next (key)) {
-        grlkey = key->data;
+        grlkey = GRLPOINTER_TO_KEYID (key->data);
         /* Skip id and source keys */
         if (grlkey == GRL_METADATA_KEY_ID ||
             grlkey == GRL_METADATA_KEY_SOURCE) {
@@ -398,7 +398,7 @@ grl_media_unserialize (const gchar *serial)
   GType type_media;
   GrlKeyID grlkey;
   GrlMedia *media;
-  GrlPluginRegistry *registry;
+  GrlRegistry *registry;
   gchar *escaped_value;
   gchar *keyname;
   gchar *protocol;
@@ -464,12 +464,12 @@ grl_media_unserialize (const gchar *serial)
   query = g_match_info_fetch (match_info, 4);
   g_match_info_free (match_info);
   if (query) {
-    registry = grl_plugin_registry_get_default ();
+    registry = grl_registry_get_default ();
     query_regex = g_regex_new ("([^=&]+)=([^=&]*)", 0, 0, NULL);
     g_regex_match (query_regex, query, 0, &match_info);
     while (g_match_info_matches (match_info)) {
       keyname = g_match_info_fetch (match_info, 1);
-      grlkey = grl_plugin_registry_lookup_metadata_key (registry, keyname);
+      grlkey = grl_registry_lookup_metadata_key (registry, keyname);
       if (grlkey) {
         escaped_value = g_match_info_fetch (match_info, 2);
         value = g_uri_unescape_string (escaped_value, NULL);
@@ -672,18 +672,20 @@ grl_media_set_duration (GrlMedia *media, gint duration)
 }
 
 /**
- * grl_media_set_date:
+ * grl_media_set_publication_date:
  * @media: the media
  * @date: the date
  *
- * Set the media's date (TBD)
+ * Set the publication date of @media.
+ *
+ * Since: 0.2.0
  */
 void
-grl_media_set_date (GrlMedia *media, const gchar *date)
+grl_media_set_publication_date (GrlMedia *media, const GDateTime *date)
 {
-  grl_data_set_string (GRL_DATA (media),
-                       GRL_METADATA_KEY_DATE,
-                       date);
+  grl_data_set_boxed (GRL_DATA (media),
+                      GRL_METADATA_KEY_PUBLICATION_DATE,
+                      date);
 }
 
 /**
@@ -693,15 +695,34 @@ grl_media_set_date (GrlMedia *media, const gchar *date)
   *
   * Set the creation_date of the media
   *
-  * Since: 0.1.12
+  * Since: 0.2.0
   */
 void
 grl_media_set_creation_date (GrlMedia *media,
-                             const gchar * creation_date)
+                             const GDateTime *creation_date)
 {
-  grl_data_set_string (GRL_DATA (media),
-                       GRL_METADATA_KEY_CREATION_DATE,
-                       creation_date);
+  grl_data_set_boxed (GRL_DATA (media),
+                      GRL_METADATA_KEY_CREATION_DATE,
+                      creation_date);
+}
+
+/**
+  * grl_media_set_modification_date:
+  * @media: the media
+  * @modification_date: date when the media was last modified
+  *
+  * Set the modification date of the media
+  *
+  * Since: 0.2.0
+  */
+void
+grl_media_set_modification_date (GrlMedia *media,
+                                 const GDateTime *modification_date)
+
+{
+  grl_data_set_boxed (GRL_DATA (media),
+                      GRL_METADATA_KEY_MODIFICATION_DATE,
+                      modification_date);
 }
 
 /**
@@ -857,21 +878,6 @@ grl_media_set_license (GrlMedia *media, const gchar *license)
                        license);
 }
 
-/**
- * grl_media_get_start_time:
- * @media: the media object
- *
- * Returns: the start time of the logical media resource inside the
- *          file containing it, in seconds.
- *
- * Since: 0.1.19
- */
-gfloat
-grl_media_get_start_time (GrlMedia *media)
-{
-  return grl_data_get_float (GRL_DATA (media), GRL_METADATA_KEY_START_TIME);
-}
-
 /**
  * grl_media_get_id:
  * @media: the media object
@@ -1135,32 +1141,48 @@ grl_media_get_duration (GrlMedia *media)
 }
 
 /**
- * grl_media_get_date:
+ * grl_media_get_publication_date:
  * @media: the media object
  *
- * Returns: the media's date (TBD)
+ * Returns: (transfer none): the publication date of @media (owned by @media).
  *
- * Since: 0.1.4
+ * Since: 0.2.0
  */
-const gchar *
-grl_media_get_date (GrlMedia *media)
+GDateTime *
+grl_media_get_publication_date (GrlMedia *media)
 {
-  return grl_data_get_string (GRL_DATA (media), GRL_METADATA_KEY_DATE);
+  return grl_data_get_boxed (GRL_DATA (media),
+                             GRL_METADATA_KEY_PUBLICATION_DATE);
 }
 
 /**
  * grl_media_get_creation_date:
  * @media: the media
  *
- * Returns: date when media was created
+ * Returns: (transfer none): date when media was created (owned by @media).
  *
- * Since: 0.1.12
+ * Since: 0.2.0
  */
-const gchar *
+GDateTime *
 grl_media_get_creation_date (GrlMedia *media)
 {
-  return grl_data_get_string (GRL_DATA (media),
-                              GRL_METADATA_KEY_CREATION_DATE);
+  return grl_data_get_boxed (GRL_DATA (media), GRL_METADATA_KEY_CREATION_DATE);
+}
+
+/**
+ * grl_media_get_modification_date:
+ * @media: the media
+ *
+ * Returns: (transfer none):date when the media was last modified (owned by @media).
+ *
+ * Since: 0.2.0
+ */
+GDateTime *
+grl_media_get_modification_date (GrlMedia *media)
+
+{
+  return grl_data_get_boxed (GRL_DATA (media),
+                             GRL_METADATA_KEY_MODIFICATION_DATE);
 }
 
 /**
@@ -1354,3 +1376,18 @@ grl_media_get_license (GrlMedia *media)
 {
   return grl_data_get_string (GRL_DATA (media), GRL_METADATA_KEY_LICENSE);
 }
+
+/**
+ * grl_media_get_start_time:
+ * @media: the media object
+ *
+ * Returns: the start time of the logical media resource inside the
+ *          file containing it, in seconds.
+ *
+ * Since: 0.1.19
+ */
+gfloat
+grl_media_get_start_time (GrlMedia *media)
+{
+  return grl_data_get_float (GRL_DATA (media), GRL_METADATA_KEY_START_TIME);
+}
