@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 Igalia S.L.
+ * Copyright (C) 2010, 2011 Igalia S.L.
  *
  * Contact: Iago Toral Quiroga <itoral@igalia.com>
  *
@@ -23,7 +23,7 @@
 /**
  * SECTION:grl-multiple
  * @short_description: Search in multiple loaded sources
- * @see_also: #GrlMediaPlugin, #GrlMetadataSource, #GrlMediaSource
+ * @see_also: #GrlPlugin, #GrlSource
  *
  * These helper functions are due to ease the search in multiple sources.
  * You can specify the list of sources to use for the searching. Those sources
@@ -37,7 +37,7 @@
 #include "grl-sync-priv.h"
 #include "grl-operation.h"
 #include "grl-operation-priv.h"
-#include "grl-plugin-registry.h"
+#include "grl-registry.h"
 #include "grl-error.h"
 #include "grl-log.h"
 
@@ -57,8 +57,8 @@ struct MultipleSearchData {
   guint sources_count;
   GList *sources_more;
   gchar *text;
-  GrlMetadataResolutionFlags flags;
-  GrlMediaSourceResultCb user_callback;
+  GrlOperationOptions *options;
+  GrlSourceResultCb user_callback;
   gpointer user_data;
 };
 
@@ -70,27 +70,25 @@ struct ResultCount {
 };
 
 struct CallbackData {
-  GrlMediaSourceResultCb user_callback;
+  GrlSourceResultCb user_callback;
   gpointer user_data;
 };
 
 struct MediaFromUriCallbackData {
   gchar *uri;
-  GrlMediaSourceMetadataCb user_callback;
+  GrlSourceResolveCb user_callback;
   gpointer user_data;
 };
 
-static void multiple_search_cb (GrlMediaSource *source,
-				guint search_id,
-				GrlMedia *media,
-				guint remaining,
-				gpointer user_data,
-				const GError *error);
+static void multiple_search_cb (GrlSource *source,
+                                guint search_id,
+                                GrlMedia *media,
+                                guint remaining,
+                                gpointer user_data,
+                                const GError *error);
+
 static void multiple_search_cancel_cb (struct MultipleSearchData *msd);
 
-
-/* ================= Globals ================= */
-
 /* ================ Utitilies ================ */
 
 static void
@@ -102,6 +100,7 @@ free_multiple_search_data (struct MultipleSearchData *msd)
   g_list_free (msd->sources);
   g_list_free (msd->sources_more);
   g_list_free (msd->keys);
+  g_object_unref (msd->options);
   g_free (msd->text);
   g_free (msd);
 }
@@ -131,7 +130,7 @@ handle_no_searchable_sources_idle (gpointer user_data)
 }
 
 static void
-handle_no_searchable_sources (GrlMediaSourceResultCb callback, gpointer user_data)
+handle_no_searchable_sources (GrlSourceResultCb callback, gpointer user_data)
 {
   struct CallbackData *callback_data = g_new0 (struct CallbackData, 1);
   callback_data->user_callback = callback;
@@ -145,52 +144,62 @@ start_multiple_search_operation (guint search_id,
 				 const gchar *text,
 				 const GList *keys,
 				 const GList *skip_counts,
-				 guint count,
-				 GrlMetadataResolutionFlags flags,
-				 GrlMediaSourceResultCb user_callback,
+				 gint count,
+				 GrlOperationOptions *options,
+				 GrlSourceResultCb user_callback,
 				 gpointer user_data)
 {
   GRL_DEBUG ("start_multiple_search_operation");
 
   struct MultipleSearchData *msd;
   GList *iter_sources, *iter_skips;
-  guint n, first_count, individual_count;
+  guint n;
+  gint first_count, individual_count;
 
   /* Prepare data required to execute the operation */
   msd = g_new0 (struct MultipleSearchData, 1);
   msd->table = g_hash_table_new_full (g_direct_hash, g_direct_equal,
 				      NULL, g_free);
-  msd->remaining = count - 1;
+  msd->remaining =
+      (count == GRL_COUNT_INFINITY) ? GRL_COUNT_INFINITY : (count - 1);
   msd->search_id = search_id;
   msd->text = g_strdup (text);
   msd->keys = g_list_copy ((GList *) keys);
-  msd->flags = flags;
+  msd->options = g_object_ref (options);
   msd->user_callback = user_callback;
   msd->user_data = user_data;
 
   /* Compute the # of items to request by each source */
   n = g_list_length ((GList *) sources);
-  individual_count = count / n;
-  first_count = individual_count + count % n;
+  if (count == GRL_COUNT_INFINITY) {
+    individual_count = GRL_COUNT_INFINITY;
+    first_count = GRL_COUNT_INFINITY;
+  } else {
+    individual_count = count / n;
+    first_count = individual_count + count % n;
+  }
 
   /* Issue search operations on each source */
   iter_sources = (GList *) sources;
   iter_skips = (GList *) skip_counts;
   n = 0;
   while (iter_sources) {
-    GrlMediaSource *source;
+    GrlSource *source;
     guint c, id;
     struct ResultCount *rc;
     guint skip;
 
-    source = GRL_MEDIA_SOURCE (iter_sources->data);
+    source = GRL_SOURCE (iter_sources->data);
 
     /* c is the count to use for this source */
     c = (n == 0) ? first_count : individual_count;
     n++;
 
-    /* Only interested in sourcs with c > 0 */
-    if (c > 0) {
+    /* Only interested in sources with c != 0 */
+    if (c != 0) {
+      GrlOperationOptions *source_options = NULL;
+      GrlCaps *source_caps;
+
       /* We use ResultCount to keep track of results emitted by this source */
       rc = g_new0 (struct ResultCount, 1);
       rc->count = c;
@@ -204,19 +213,25 @@ start_multiple_search_operation (guint search_id,
 	skip = 0;
       }
 
+      source_caps = grl_source_get_caps (source, GRL_OP_SEARCH);
+      grl_operation_options_obey_caps (options, source_caps, &source_options, NULL);
+      grl_operation_options_set_skip (source_options, skip);
+      grl_operation_options_set_count (source_options, rc->count);
+
       /* Execute the search on this source */
-      id = grl_media_source_search (source,
+      id = grl_source_search (source,
 				    msd->text,
 				    msd->keys,
-				    skip, rc->count,
-				    flags,
+				    source_options,
 				    multiple_search_cb,
 				    msd);
 
       GRL_DEBUG ("Operation %s:%u: Searching %u items from offset %u",
-                 grl_metadata_source_get_name (GRL_METADATA_SOURCE (source)),
+                 grl_source_get_name (GRL_SOURCE (source)),
                  id, rc->count, skip);
 
+      g_object_unref (source_options);
+
       /* Keep track of this operation and this source */
       msd->search_ids = g_list_prepend (msd->search_ids, GINT_TO_POINTER (id));
       msd->sources = g_list_prepend (msd->sources, source);
@@ -243,14 +258,14 @@ chain_multiple_search_operation (struct MultipleSearchData *old_msd)
   GList *skip_list = NULL;
   GList *source_iter;
   struct ResultCount *rc;
-  GrlMediaSource *source;
+  GrlSource *source;
   struct MultipleSearchData *msd;
 
   /* Compute skip parameter for each of the sources that can still
      provide more results */
   source_iter = old_msd->sources_more;
   while (source_iter) {
-    source = GRL_MEDIA_SOURCE (source_iter->data);
+    source = GRL_SOURCE (source_iter->data);
     rc = (struct ResultCount *)
       g_hash_table_lookup (old_msd->table, (gpointer) source);
     skip_list = g_list_prepend (skip_list,
@@ -268,7 +283,7 @@ chain_multiple_search_operation (struct MultipleSearchData *old_msd)
 					 old_msd->keys,
 					 skip_list,
 					 old_msd->pending,
-					 old_msd->flags,
+					 old_msd->options,
 					 old_msd->user_callback,
 					 old_msd->user_data);
   g_list_free (skip_list);
@@ -277,7 +292,7 @@ chain_multiple_search_operation (struct MultipleSearchData *old_msd)
 }
 
 static void
-multiple_result_async_cb (GrlMediaSource *source,
+multiple_result_async_cb (GrlSource *source,
                           guint op_id,
                           GrlMedia *media,
                           guint remaining,
@@ -310,14 +325,14 @@ multiple_result_async_cb (GrlMediaSource *source,
 }
 
 static void
-multiple_search_cb (GrlMediaSource *source,
+multiple_search_cb (GrlSource *source,
 		    guint search_id,
 		    GrlMedia *media,
 		    guint remaining,
 		    gpointer user_data,
 		    const GError *error)
 {
-  GRL_DEBUG ("multiple_search_cb");
+  GRL_DEBUG (__FUNCTION__);
 
   struct MultipleSearchData *msd;
   gboolean emit;
@@ -328,7 +343,7 @@ multiple_search_cb (GrlMediaSource *source,
 
   GRL_DEBUG ("multiple:remaining == %u, source:remaining = %u (%s)",
              msd->remaining, remaining,
-             grl_metadata_source_get_name (GRL_METADATA_SOURCE (source)));
+             grl_source_get_name (GRL_SOURCE (source)));
 
   /* Check if operation is done, that is, if all the sources involved
      in the multiple operation have emitted remaining=0 */
@@ -374,13 +389,14 @@ multiple_search_cb (GrlMediaSource *source,
     /* This source failed to provide as many results as we requested,
        we will have to check if other sources can provide the missing
        results */
-    msd->pending += rc->count - rc->received;
+    if (rc->count != GRL_COUNT_INFINITY)
+      msd->pending += rc->count - rc->received;
   } else if (remaining == 0) {
     /* This source provided all requested results, if others did not
        we can use this to request more */
     msd->sources_more = g_list_prepend (msd->sources_more, source);
     GRL_DEBUG ("Source %s provided all requested results",
-               grl_metadata_source_get_name (GRL_METADATA_SOURCE (source)));
+               grl_source_get_name (GRL_SOURCE (source)));
   }
 
   /* --- Manage NULL results --- */
@@ -434,7 +450,6 @@ multiple_search_cb (GrlMediaSource *source,
 
  operation_done:
   GRL_DEBUG ("Multiple operation finished (%u)", msd->search_id);
-
   grl_operation_remove (msd->search_id);
 }
 
@@ -447,11 +462,11 @@ free_media_from_uri_data (struct MediaFromUriCallbackData *mfucd)
 }
 
 static void
-media_from_uri_cb (GrlMediaSource *source,
+media_from_uri_cb (GrlSource *source,
                    guint operation_id,
-		   GrlMedia *media,
-		   gpointer user_data,
-		   const GError *error)
+				   GrlMedia *media,
+				   gpointer user_data,
+				   const GError *error)
 {
   struct MediaFromUriCallbackData *mfucd =
     (struct MediaFromUriCallbackData *) user_data;
@@ -477,38 +492,36 @@ media_from_uri_cb (GrlMediaSource *source,
 
 /**
  * grl_multiple_search:
- * @sources: (element-type Grl.MediaSource) (allow-none):
- * a #GList of #GrlMediaSource<!-- -->s to search from (%NULL for all
+ * @sources: (element-type Grl.Source) (allow-none):
+ * a #GList of #GrlSource<!-- -->s to search from (%NULL for all
  * searchable sources)
  * @text: the text to search for
- * @keys: (element-type GObject.ParamSpec): the #GList of
+ * @keys: (element-type GrlKeyID): the #GList of
  * #GrlKeyID to retrieve
- * @count: the maximum number of elements to retrieve
- * @flags: the operation flags
+ * @options: options wanted for that operation
  * @callback: (scope notified): the user defined callback
  * @user_data: the user data to pass to the user callback
  *
  * Search for @text in all the sources specified in @sources.
  *
  * If @text is @NULL then NULL-text searchs will be used for each searchable
- * plugin (see #grl_media_source_search for more details).
+ * plugin (see #grl_source_search for more details).
  *
  * This method is asynchronous.
  *
  * Returns: the operation identifier
  *
- * Since: 0.1.6
+ * Since: 0.2.0
  */
 guint
 grl_multiple_search (const GList *sources,
 		     const gchar *text,
 		     const GList *keys,
-		     guint count,
-		     GrlMetadataResolutionFlags flags,
-		     GrlMediaSourceResultCb callback,
+		     GrlOperationOptions *options,
+		     GrlSourceResultCb callback,
 		     gpointer user_data)
 {
-  GrlPluginRegistry *registry;
+  GrlRegistry *registry;
   GList *sources_list;
   struct MultipleSearchData *msd;
   gboolean allocated_sources_list = FALSE;
@@ -516,17 +529,17 @@ grl_multiple_search (const GList *sources,
 
   GRL_DEBUG ("grl_multiple_search");
 
-  g_return_val_if_fail (count > 0, 0);
   g_return_val_if_fail (callback != NULL, 0);
+  g_return_val_if_fail (GRL_IS_OPERATION_OPTIONS (options), 0);
 
   /* If no sources have been provided then get the list of all
      searchable sources from the registry */
   if (!sources) {
-    registry = grl_plugin_registry_get_default ();
+    registry = grl_registry_get_default ();
     sources_list =
-      grl_plugin_registry_get_sources_by_operations (registry,
-						     GRL_OP_SEARCH,
-						     TRUE);
+      grl_registry_get_sources_by_operations (registry,
+                                              GRL_OP_SEARCH,
+                                              TRUE);
     if (sources_list == NULL) {
       /* No searchable sources? Raise error and bail out */
       g_list_free (sources_list);
@@ -545,8 +558,8 @@ grl_multiple_search (const GList *sources,
 					 text,
 					 keys,
 					 NULL,
-					 count,
-					 flags,
+					 grl_operation_options_get_count (options),
+					 options,
 					 callback,
 					 user_data);
   if  (allocated_sources_list) {
@@ -567,7 +580,7 @@ multiple_search_cancel_cb (struct MultipleSearchData *msd)
   ids = msd->search_ids;
   while (sources) {
     GRL_DEBUG ("cancelling operation %s:%u",
-               grl_metadata_source_get_name (GRL_METADATA_SOURCE (sources->data)),
+               grl_source_get_name (GRL_SOURCE (sources->data)),
                GPOINTER_TO_UINT (ids->data));
     grl_operation_cancel (GPOINTER_TO_INT (ids->data));
     sources = g_list_next (sources);
@@ -580,37 +593,15 @@ multiple_search_cancel_cb (struct MultipleSearchData *msd)
   g_idle_add (confirm_cancel_idle, msd);
 }
 
-/**
- * grl_multiple_cancel:
- * @search_id: the identifier of the multiple operation to cancel
- *
- * Cancel a running multiple search by issuing a cancel operation on each
- * source involved involved in the operation.
- *
- * Since: 0.1.6
- * Deprecated: 0.1.16: use grl_operation_cancel() instead.
- */
-void
-grl_multiple_cancel (guint search_id)
-{
-  GRL_DEBUG ("grl_multiple_cancel");
-
-  GRL_WARNING ("grl_multiple_cancel() is deprecated. "
-               "Use grl_operation_cancel() instead");
-
-  grl_operation_cancel (search_id);
-}
-
 /**
  * grl_multiple_search_sync:
- * @sources: (element-type Grl.MediaSource) (allow-none):
- * a #GList of #GrlMediaSource<!-- -->s where to search from (%NULL for all
+ * @sources: (element-type Grl.Source) (allow-none):
+ * a #GList of #GrlSource<!-- -->s where to search from (%NULL for all
  * available sources with search capability)
  * @text: the text to search for
- * @keys: (element-type GObject.ParamSpec): the #GList of
+ * @keys: (element-type GrlKeyID): the #GList of
  * #GrlKeyID to retrieve
- * @count: the maximum number of elements to retrieve
- * @flags: the operation flags
+ * @options: options wanted for that operation
  * @error: a #GError, or @NULL
  *
  * Search for @text in all the sources specified in @sources.
@@ -619,14 +610,13 @@ grl_multiple_cancel (guint search_id)
  *
  * Returns: (element-type Grl.Media) (transfer full): a list with #GrlMedia elements
  *
- * Since: 0.1.6
+ * Since: 0.2.0
  */
 GList *
 grl_multiple_search_sync (const GList *sources,
                           const gchar *text,
                           const GList *keys,
-                          guint count,
-                          GrlMetadataResolutionFlags flags,
+                          GrlOperationOptions *options,
                           GError **error)
 {
   GrlDataSync *ds;
@@ -637,8 +627,7 @@ grl_multiple_search_sync (const GList *sources,
   grl_multiple_search (sources,
                        text,
                        keys,
-                       count,
-                       flags,
+                       options,
                        multiple_result_async_cb,
                        ds);
 
@@ -662,7 +651,7 @@ grl_multiple_search_sync (const GList *sources,
  * grl_multiple_get_media_from_uri:
  * @uri: A URI that can be used to identify a media resource
  * @keys: (element-type GrlKeyID): List of metadata keys we want to obtain.
- * @flags: the operation flags
+ * @options: options wanted for that operation
  * @callback: (scope notified): the user defined callback
  * @user_data: the user data to pass to the user callback
  *
@@ -672,34 +661,35 @@ grl_multiple_search_sync (const GList *sources,
  *
  * This method is asynchronous.
  *
- * Since: 0.1.7
+ * Since: 0.2.0
  */
 void
 grl_multiple_get_media_from_uri (const gchar *uri,
-				 const GList *keys,
-				 GrlMetadataResolutionFlags flags,
-				 GrlMediaSourceMetadataCb callback,
-				 gpointer user_data)
+				      const GList *keys,
+				      GrlOperationOptions *options,
+				      GrlSourceResolveCb callback,
+				      gpointer user_data)
 {
-  GrlPluginRegistry *registry;
+  GrlRegistry *registry;
   GList *sources, *iter;
   gboolean found = FALSE;
 
   g_return_if_fail (uri != NULL);
   g_return_if_fail (keys != NULL);
   g_return_if_fail (callback != NULL);
+  g_return_if_fail (GRL_IS_OPERATION_OPTIONS (options));
 
-  registry = grl_plugin_registry_get_default ();
+  registry = grl_registry_get_default ();
   sources =
-    grl_plugin_registry_get_sources_by_operations (registry,
-						   GRL_OP_MEDIA_FROM_URI,
-						   TRUE);
+    grl_registry_get_sources_by_operations (registry,
+                                            GRL_OP_MEDIA_FROM_URI,
+                                            TRUE);
 
   /* Look for the first source that knows how to deal with 'uri' */
   iter = sources;
   while (iter && !found) {
-    GrlMediaSource *source = GRL_MEDIA_SOURCE (iter->data);
-    if (grl_media_source_test_media_from_uri (source, uri)) {
+    GrlSource *source = GRL_SOURCE (iter->data);
+    if (grl_source_test_media_from_uri (source, uri)) {
       struct MediaFromUriCallbackData *mfucd =
 	g_new0 (struct MediaFromUriCallbackData, 1);
 
@@ -707,10 +697,10 @@ grl_multiple_get_media_from_uri (const gchar *uri,
       mfucd->user_data = user_data;
       mfucd->uri = g_strdup (uri);
 
-      grl_media_source_get_media_from_uri (source,
+      grl_source_get_media_from_uri (source,
 					   uri,
 					   keys,
-					   flags,
+					   options,
 					   media_from_uri_cb,
 					   mfucd);
       found = TRUE;
